@@ -55,6 +55,8 @@ class DownstreamInfoDriver(PkgInfoDriver):
             'downstream_distro_branch': {},
             'downstream_tag': {},
             'downstream_distgit_key': {},
+            'downstream_source_git_key': {},
+            'downstream_source_git_branch': {},
             'use_upstream_spec': {'type': 'boolean'},
             'downstream_spec_replace_list': {'type': 'list'},
             'cache_dir': {},
@@ -218,20 +220,36 @@ class DownstreamInfoDriver(PkgInfoDriver):
         ups_distro_dir_full = self.upstream_distgit_dir(package['name'])
         ups_distro_branch = getdistrobranch(package)
 
+        # Downstream source git
+        dsgit_attr = self.config_options.downstream_source_git_key
+        ds_source = package.get(dsgit_attr)
+        if not distro:
+            fail_req_attr_missing(dsgit_attr, package)
+        dsgit_dir = self._downstream_git_clone_dir(package['name'])
+        dsgit_branch = self.config_options.downstream_source_git_branch
+
         # only process packages present in versions.csv
         if package['name'] not in versions:
             logger.warning('Package %s not present in %s - skipping.' % (
                 package['name'], self.config_options.versions_url))
-            return []
+            return PkgInfoDriver.Info([], True)
         version = versions[package['name']]
 
         dt_distro = 0  # In this driver we do not care about dt_distro
 
         if dev_mode is False:
             try:
-                distro_branch, extended_hash, dt_extended = refreshrepo(
+                distro_branch, extended_hash_dsdist, dt_extended = refreshrepo(
                     distro, distro_dir, distro_branch, local=local,
                     full_path=distro_dir_full)
+
+                _, extended_hash_dssource, _ = refreshrepo(
+                    ds_source, dsgit_dir, dsgit_branch, local=local,
+                    full_path=dsgit_dir)
+
+                extended_hash = '%s_%s' % (extended_hash_dsdist,
+                                           extended_hash_dssource)
+
                 # extract distro_hash from versions.csv
                 distro_hash = version[3]
                 # Also download upstream distgit
@@ -243,7 +261,7 @@ class DownstreamInfoDriver(PkgInfoDriver):
                 # The error was already logged by refreshrepo, and we want
                 # to avoid halting the whole run because this distgit repo
                 # failed, so return an empty list
-                return []
+                return PkgInfoDriver.Info([], True)
         else:
             distro_hash = "dev"
             extended_hash = "dev"
@@ -276,7 +294,7 @@ class DownstreamInfoDriver(PkgInfoDriver):
                 # side-effect is that we are not adding this commit to the
                 # list of commits to be processed, so we can ignore it and
                 # move on to the next repo
-                continue
+                return PkgInfoDriver.Info([], True)
             if not local:
                 # This is the default behavior
                 dt = version[5]
@@ -307,7 +325,7 @@ class DownstreamInfoDriver(PkgInfoDriver):
                             commit_branch=source_branch, component=component)
             project_toprocess.append(commit)
 
-        return project_toprocess
+        return PkgInfoDriver.Info(project_toprocess, False)
 
     def preprocess(self, **kwargs):
         package_name = kwargs.get('package_name')
@@ -379,6 +397,10 @@ class DownstreamInfoDriver(PkgInfoDriver):
         datadir = self.config_options.datadir
         return os.path.join(datadir, package_name + "_distro")
 
+    def _downstream_git_clone_dir(self, package_name):
+        datadir = self.config_options.datadir
+        return os.path.join(datadir, package_name + "_downstream")
+
     def upstream_distgit_dir(self, package_name):
         datadir = self.config_options.datadir
         # Find extra directory inside it, if needed
